@@ -4,32 +4,41 @@ import re
 import itertools
 import operator
 
-from urlparse import urljoin, urlparse
+# Python 2 and 3
+import sys
+if sys.version_info >= (3, 0):
+    from urllib.parse import urlparse
+else:
+    from urlparse import urlparse
+
 from threading import Lock
 from collections import namedtuple
-from collections import defaultdict
+from collections import OrderedDict
 
 from .externals.reqresp import Request, Response
-from .exception import FuzzExceptBadAPI, FuzzExceptBadOptions, FuzzExceptInternalError, FuzzExceptBadInstall
+from .exception import FuzzExceptBadAPI, FuzzExceptBadOptions, FuzzExceptInternalError
 from .facade import Facade
 from .mixins import FuzzRequestUrlMixing, FuzzRequestSoupMixing
 
+from .utils import python2_3_convert_to_unicode
+
 auth_header = namedtuple("auth_header", "method credentials")
 
+
 class headers:
     def __init__(self, req):
         self._req = req
 
     @property
     def response(self):
-        return dict(self._req.response.getHeaders()) if self._req.response else {}
+        return OrderedDict(self._req.response.getHeaders()) if self._req.response else {}
 
     @property
     def request(self):
-        return dict(map(lambda x: x.split(": ",1), self._req.getHeaders()))
+        return OrderedDict([x.split(": ", 1) for x in self._req.getHeaders()])
 
     def add(self, dd):
-        for k, v in dd.items():
+        for k, v in list(dd.items()):
             self._req._headers[k] = v
 
     def get_field(self, field):
@@ -38,9 +47,9 @@ class headers:
 
         if num_fields == 2:
             if attr[1] == "request":
-                return ", ".join(map(lambda x: "%s:%s" % (x[0],x[1]), self.request.items()))
+                return ", ".join(["%s:%s" % (x[0], x[1]) for x in list(self.request.items())])
             elif attr[1] == "response":
-                return ", ".join(map(lambda x: "%s:%s" % (x[0],x[1]), self.response.items()))
+                return ", ".join(["%s:%s" % (x[0], x[1]) for x in list(self.response.items())])
             else:
                 raise FuzzExceptBadAPI("headers must be specified in the form of headers.[request|response].<header name>")
         elif num_fields != 3:
@@ -59,6 +68,7 @@ class headers:
 
         return ret.strip()
 
+
 class cookies:
     def __init__(self, req):
         self._req = req
@@ -68,20 +78,16 @@ class cookies:
         if self._req.response:
             c = self._req.response.getCookie().split("; ")
             if c[0]:
-                #cc['response'] = dict(map(lambda x: x.split("=", 1), c))
-                return dict(map(lambda x:[x[0],x[2]],map(lambda x:x.partition("="), c)))
+                return OrderedDict([[x[0], x[2]] for x in [x.partition("=") for x in c]])
 
         return {}
 
-
     @property
     def request(self):
         if 'Cookie' in self._req._headers:
-            #.request['COOKIE'].split("; ")
             c = self._req._headers['Cookie'].split("; ")
             if c[0]:
-                #cc['request'] = dict(map(lambda x: x.split("=", 1), c))
-                return dict(map(lambda x:[x[0],x[2]],map(lambda x:x.partition("="), c)))
+                return OrderedDict([[x[0], x[2]] for x in [x.partition("=") for x in c]])
 
         return {}
 
@@ -114,22 +120,23 @@ class cookies:
 
         return ""
 
+
 class params(object):
     def __init__(self, req):
         self._req = req
 
     @property
     def get(self):
-        return dict(map(lambda x: (x.name, x.value), self._req.getGETVars()))
+        return OrderedDict([(x.name, x.value) for x in self._req.getGETVars()])
 
     @property
     def post(self):
-        return dict(map(lambda x: (x.name, x.value), self._req.getPOSTVars()))
+        return OrderedDict([(x.name, x.value) for x in self._req.getPOSTVars()])
 
     @post.setter
     def post(self, pp):
         if isinstance(pp, dict):
-            self._req.setPostData("&".join(["=".join([n,v]) if v is not None else n for n,v in pp.items()]))
+            self._req.setPostData("&".join(["=".join([n, v]) if v is not None else n for n, v in list(pp.items())]))
         elif isinstance(pp, str):
             self._req.setPostData(pp)
 
@@ -138,13 +145,13 @@ class params(object):
         num_fields = len(attr)
 
         if num_fields == 1 and attr[0] == "params":
-                pp = ", ".join(map(lambda x: "%s:%s" % (x[0],x[1]), dict(self.get.items() + self.post.items()).items()))
+                pp = ", ".join(["%s:%s" % (x[0], x[1]) for x in list(dict(list(self.get.items()) + list(self.post.items())).items())])
                 return "" if not pp else pp
         elif num_fields == 2:
             if attr[1] == "get":
-                return ", ".join(map(lambda x: "%s=%s" % (x[0],x[1]), self.get.items()))
+                return ", ".join(["%s=%s" % (x[0], x[1]) for x in list(self.get.items())])
             elif attr[1] == "post":
-                return ", ".join(map(lambda x: "%s=%s" % (x[0],x[1]), self.post.items()))
+                return ", ".join(["%s=%s" % (x[0], x[1]) for x in list(self.post.items())])
             else:
                 raise FuzzExceptBadAPI("Parameters must be specified as params.[get/post].<name>")
         elif num_fields == 3:
@@ -163,16 +170,17 @@ class params(object):
         else:
             raise FuzzExceptBadAPI("Parameters must be specified as params.[get/post].<name>")
 
-class FuzzRequest(object, FuzzRequestUrlMixing, FuzzRequestSoupMixing):
-    def __init__(self):
-	self._request = Request()
 
-	self._proxy = None
-	self._allvars = None
-	self.wf_fuzz_methods = None
+class FuzzRequest(FuzzRequestUrlMixing, FuzzRequestSoupMixing):
+    def __init__(self):
+        self._request = Request()
+
+        self._proxy = None
+        self._allvars = None
+        self.wf_fuzz_methods = None
         self.wf_retries = 0
 
-	self.headers.add({"User-Agent": Facade().sett.get("connection","User-Agent").encode('utf-8')})
+        self.headers.add({"User-Agent": Facade().sett.get("connection", "user-agent")})
 
     # methods for accessing HTTP requests information consistenly accross the codebase
 
@@ -192,96 +200,97 @@ class FuzzRequest(object, FuzzRequestUrlMixing, FuzzRequestSoupMixing):
 
     @property
     def params(self):
-	return params(self._request)
+        return params(self._request)
 
     @property
     def cookies(self):
-	return cookies(self._request)
+        return cookies(self._request)
 
     @property
     def method(self):
-	return self._request.method
+        return self._request.method
 
     @method.setter
     def method(self, method):
-	self._request.method = method
+        self._request.method = method
 
     @property
     def scheme(self):
-	return self._request.schema
+        return self._request.schema
 
     @scheme.setter
     def scheme(self, s):
-	self._request.schema = s
+        self._request.schema = s
 
     @property
     def host(self):
-	return self._request.getHost()
+        return self._request.getHost()
 
     @property
     def path(self):
-	return self._request.path
+        return self._request.path
 
     @property
     def redirect_url(self):
-	return self._request.completeUrl
+        return self._request.completeUrl
 
     @property
     def url(self):
-	return self._request.finalUrl
+        return self._request.finalUrl
 
     @url.setter
     def url(self, u):
-        if urlparse(u).path == "":
-            u += '/'
-
         if not u.startswith("FUZ") and urlparse(u).scheme == "":
             u = "http://" + u
 
-        if Facade().sett.get("general","encode_space") == "1":
+        if urlparse(u).path == "":
+            u += '/'
+
+        if Facade().sett.get("general", "encode_space") == "1":
             u = u.replace(" ", "%20")
 
-	self._request.setUrl(u)
+        self._request.setUrl(u)
         if self.scheme.startswith("fuz") and self.scheme.endswith("z"):
-            self.scheme = self.scheme.upper() # avoid FUZZ to become fuzz
+            # avoid FUZZ to become fuzz
+            self.scheme = self.scheme.upper()
 
     @property
     def content(self):
-	return self._request.response.getContent() if self._request.response else ""
+        return self._request.response.getContent() if self._request.response else ""
 
     @property
     def code(self):
-	return self._request.response.code if self._request.response else 0
+        return self._request.response.code if self._request.response else 0
 
     @code.setter
     def code(self, c):
-	self._request.response.code = c
+        self._request.response.code = c
 
     @property
     def auth(self):
-	m, up = self._request.getAuth()
+        m, up = self._request.getAuth()
         return auth_header(m, up)
 
     @auth.setter
     def auth(self, ah):
         method, credentials = ah
-	self._request.setAuth(method, credentials)
+        self._request.setAuth(method, credentials)
 
     @property
     def follow(self):
-	return self._request.followLocation
+        return self._request.followLocation
 
     @follow.setter
     def follow(self, f):
-	self._request.setFollowLocation(f)
+        self._request.setFollowLocation(f)
 
     @property
     def reqtime(self):
-	return self._request.totaltime
+        return self._request.totaltime
 
     @reqtime.setter
     def reqtime(self, t):
-	self._request.totaltime = t
+        self._request.totaltime = t
 
     def set_field(self, field, value):
         if field in ["url"]:
@@ -294,6 +303,8 @@ class FuzzRequest(object, FuzzRequestUrlMixing, FuzzRequestSoupMixing):
             field = alias[field]
 
         if field in ["url", "method", "scheme", "host", "content", "raw_content", "code"]:
+            return getattr(self, field)
+        elif field in ["code"]:
             return str(getattr(self, field))
         elif field.startswith("cookies"):
             return self.cookies.get_field(field).strip()
@@ -324,12 +335,12 @@ class FuzzRequest(object, FuzzRequestUrlMixing, FuzzRequestSoupMixing):
     # Info extra that wfuzz needs within an HTTP request
     @property
     def wf_allvars_set(self):
-	if self.wf_allvars == "allvars":
-	    return self._request.getGETVars()
-	elif self.wf_allvars == "allpost":
-	    return self._request.getPOSTVars()
-	elif self.wf_allvars == "allheaders":
-	    return self._request.getHeaders()
+        if self.wf_allvars == "allvars":
+            return self._request.getGETVars()
+        elif self.wf_allvars == "allpost":
+            return self._request.getPOSTVars()
+        elif self.wf_allvars == "allheaders":
+            return self._request.getHeaders()
         else:
             raise FuzzExceptBadOptions("Unknown variable set: " + self.wf_allvars)
 
@@ -345,70 +356,70 @@ class FuzzRequest(object, FuzzRequestUrlMixing, FuzzRequestSoupMixing):
                 self._request.headers.add({variable: payload_content})
             else:
                 raise FuzzExceptBadOptions("Unknown variable set: " + self.wf_allvars)
-        except TypeError, e:
+        except TypeError:
             raise FuzzExceptBadOptions("It is not possible to use all fuzzing with duplicated parameters.")
-        
+
     @property
     def wf_allvars(self):
-	return self._allvars
+        return self._allvars
 
     @wf_allvars.setter
     def wf_allvars(self, bl):
-	if bl is not None and bl not in ['allvars', 'allpost','allheaders']: 
-	    raise FuzzExceptBadOptions("Incorrect all parameters brute forcing type specified, correct values are allvars, allpost or allheaders.")
+        if bl is not None and bl not in ['allvars', 'allpost', 'allheaders']:
+            raise FuzzExceptBadOptions("Incorrect all parameters brute forcing type specified, correct values are allvars, allpost or allheaders.")
 
-	self._allvars = bl
+        self._allvars = bl
 
     @property
     def wf_proxy(self):
-	return self._proxy
+        return self._proxy
 
     @wf_proxy.setter
     def wf_proxy(self, l):
-	if l:
-	    prox, ptype = l
-	    self._request.setProxy("%s" % prox, ptype if ptype else "HTML")
-	self._proxy = l
+        if l:
+            prox, ptype = l
+            self._request.setProxy("%s" % prox, ptype if ptype else "HTML")
+        self._proxy = l
 
     # methods wfuzz needs to perform HTTP requests (this might change in the future).
 
     def perform(self):
-        res = FuzzResult(self, track_id = False)
+        res = FuzzResult(self, track_id=False)
         return Facade().http_pool.perform(res)
-        
+
     def to_http_object(self, c):
         return Request.to_pycurl_object(c, self._request)
 
     def from_http_object(self, c, bh, bb):
-	return self._request.response_from_conn_object(c, bh, bb)
+        return self._request.response_from_conn_object(c, bh, bb)
 
-    def update_from_raw_http(self, raw, scheme, raw_response = None):
+    def update_from_raw_http(self, raw, scheme, raw_response=None):
         self._request.parseRequest(raw, scheme)
 
         if raw_response:
-            rp=Response()
+            rp = Response()
             rp.parseResponse(raw_response)
-            self._request.response=rp
+            self._request.response = rp
 
         return self._request
 
     def to_cache_key(self):
-	key = self._request.urlWithoutVariables
+        key = self._request.urlWithoutVariables
 
-	dicc = {}
+        dicc = {}
 
-	for j in [i.name for i in self._request.getGETVars()]:
-	    dicc[j] = True
+        for j in [i.name for i in self._request.getGETVars()]:
+            dicc[j] = True
 
-	for j in [i.name for i in self._request.getPOSTVars()]:
-	    dicc[j] = True
+        for j in [i.name for i in self._request.getPOSTVars()]:
+            dicc[j] = True
 
-	# take URL parameters into consideration
-	url_params = dicc.keys()
-	url_params.sort()
-	key += "-" + "-".join(url_params)
+        # take URL parameters into consideration
+        url_params = list(dicc.keys())
+        url_params.sort()
+        key += "-" + "-".join(url_params)
 
-	return key
+        return key
 
     # methods wfuzz needs for substituing payloads and building dictionaries
 
@@ -416,11 +427,11 @@ class FuzzRequest(object, FuzzRequestUrlMixing, FuzzRequestSoupMixing):
         if options["url"] != "FUZZ":
             self.url = options["url"]
 
-	if options['auth'][0] is not None:
-	    self.auth = (options['auth'][0], options['auth'][1])
+        if options['auth'][0] is not None:
+            self.auth = (options['auth'][0], options['auth'][1])
 
-	if options['follow']:
-	    self.follow = options['follow']
+        if options['follow']:
+            self.follow = options['follow']
 
         if options['postdata']:
             self.params.post = options['postdata']
@@ -429,45 +440,45 @@ class FuzzRequest(object, FuzzRequestUrlMixing, FuzzRequestSoupMixing):
             self.method = options['method']
             self.wf_fuzz_methods = options['method']
 
-	if options['cookie']:
+        if options['cookie']:
             self.headers.add({"Cookie": "; ".join(options['cookie'])})
 
         self.headers.add(dict(options['headers']))
 
         if options['allvars']:
-	    self.wf_allvars = options['allvars']
+            self.wf_allvars = options['allvars']
 
     def from_copy(self):
-	newreq = FuzzRequest()
-
-	newreq.wf_proxy = self.wf_proxy
-	newreq.wf_allvars = self.wf_allvars
-	newreq.wf_fuzz_methods = self.wf_fuzz_methods
+        newreq = FuzzRequest()
 
+        newreq.wf_proxy = self.wf_proxy
+        newreq.wf_allvars = self.wf_allvars
+        newreq.wf_fuzz_methods = self.wf_fuzz_methods
 
         newreq.headers.add(self.headers.request)
         newreq.params.post = self.params.post
 
-	newreq.follow = self.follow
-	newreq.auth = self.auth
-	newreq.url = self.url
-	newreq.reqtime = self.reqtime
-	newreq.scheme = self.scheme
+        newreq.follow = self.follow
+        newreq.auth = self.auth
+        newreq.url = self.url
+        newreq.reqtime = self.reqtime
+        newreq.scheme = self.scheme
         newreq.method = self.wf_fuzz_methods if self.wf_fuzz_methods else self.method
 
-	return newreq
+        return newreq
+
 
 class FuzzResultFactory:
     @staticmethod
     def replace_fuzz_word(text, fuzz_word, payload):
-        marker_regex = re.compile("(%s)(?:\[(.*?)\])?" % (fuzz_word,),re.MULTILINE|re.DOTALL)
+        marker_regex = re.compile(r"(%s)(?:\[(.*?)\])?" % (fuzz_word,), re.MULTILINE | re.DOTALL)
 
         for fw, field in marker_regex.findall(text):
             if field:
-		marker_regex = re.compile("(%s)(?:\[(.*?)\])?" % (fuzz_word,),re.MULTILINE|re.DOTALL)
-		subs_array = []
+                marker_regex = re.compile(r"(%s)(?:\[(.*?)\])?" % (fuzz_word,), re.MULTILINE | re.DOTALL)
+                subs_array = []
 
-		for fw, field in marker_regex.findall(text):
+                for fw, field in marker_regex.findall(text):
                         if not field:
                             raise FuzzExceptBadOptions("You must specify a field when using a payload containing a full fuzz request, ie. FUZZ[url], or use FUZZ only to repeat the same request.")
 
@@ -476,34 +487,34 @@ class FuzzResultFactory:
                         except AttributeError:
                             raise FuzzExceptBadOptions("A FUZZ[field] expression must be used with a fuzzresult payload not a string.")
 
-			text = text.replace("%s[%s]" % (fw, field), subs)
-			subs_array.append(subs)
+                        text = text.replace("%s[%s]" % (fw, field), subs)
+                        subs_array.append(subs)
 
-		return (text, subs_array)
+                return (text, subs_array)
             else:
                 try:
                     return (text.replace(fuzz_word, payload), [payload])
-                except TypeError, e:
+                except TypeError:
                     raise FuzzExceptBadOptions("Tried to replace FUZZ with a whole fuzzresult payload.")
 
     @staticmethod
     def from_seed(seed, payload, seed_options):
-	newres = seed.from_soft_copy()
+        newres = seed.from_soft_copy()
 
-	rawReq = str(newres.history)
-	rawUrl = newres.history.redirect_url
-	scheme = newres.history.scheme
-	auth_method, userpass = newres.history.auth
+        rawReq = str(newres.history)
+        rawUrl = newres.history.redirect_url
+        scheme = newres.history.scheme
+        auth_method, userpass = newres.history.auth
 
         descr_array = []
 
-	for payload_pos, payload_content in enumerate(payload, start=1):
-	    fuzz_word = "FUZ" + str(payload_pos) + "Z" if payload_pos > 1 else "FUZZ"
+        for payload_pos, payload_content in enumerate(payload, start=1):
+            fuzz_word = "FUZ" + str(payload_pos) + "Z" if payload_pos > 1 else "FUZZ"
 
             newres.payload.append(payload_content)
 
             # substitute entire seed when using a request payload generator without specifying field
-            if (fuzz_word == "FUZZ" and (rawUrl == "FUZZ" or seed_options["seed_payload"] == True)) and isinstance(payload_content, FuzzResult):
+            if (fuzz_word == "FUZZ" and (rawUrl == "FUZZ" or seed_options["seed_payload"] is True)) and isinstance(payload_content, FuzzResult):
                 # new seed
                 newres = payload_content.from_soft_copy()
 
@@ -519,55 +530,56 @@ class FuzzResultFactory:
 
             desc = None
 
-	    if auth_method and (userpass.count(fuzz_word)):
-		userpass, desc = FuzzResultFactory.replace_fuzz_word(userpass, fuzz_word, payload_content)
-	    if newres.history.redirect_url.count(fuzz_word):
-		rawUrl, desc = FuzzResultFactory.replace_fuzz_word(rawUrl, fuzz_word, payload_content)
-	    if rawReq.count(fuzz_word):
-		rawReq, desc = FuzzResultFactory.replace_fuzz_word(rawReq, fuzz_word, payload_content)
+            if auth_method and (userpass.count(fuzz_word)):
+                userpass, desc = FuzzResultFactory.replace_fuzz_word(userpass, fuzz_word, payload_content)
+            if newres.history.redirect_url.count(fuzz_word):
+                rawUrl, desc = FuzzResultFactory.replace_fuzz_word(rawUrl, fuzz_word, payload_content)
+            if rawReq.count(fuzz_word):
+                rawReq, desc = FuzzResultFactory.replace_fuzz_word(rawReq, fuzz_word, payload_content)
 
-	    if scheme.count(fuzz_word):
-		scheme, desc = FuzzResultFactory.replace_fuzz_word(scheme, fuzz_word, payload_content)
+            if scheme.count(fuzz_word):
+                scheme, desc = FuzzResultFactory.replace_fuzz_word(scheme, fuzz_word, payload_content)
 
             if desc:
                 descr_array += desc
 
             if len(descr_array) == 0:
-		raise FuzzExceptBadOptions("No %s word!" % fuzz_word)
+                raise FuzzExceptBadOptions("No %s word!" % fuzz_word)
 
-	newres.history.update_from_raw_http(rawReq, scheme)
-	newres.history.url = rawUrl
-	if auth_method != 'None': newres.history.auth = (auth_method, userpass)
+        newres.history.update_from_raw_http(rawReq, scheme)
+        newres.history.url = rawUrl
+        if auth_method != 'None':
+            newres.history.auth = (auth_method, userpass)
 
         if newres._description:
             newres._description += " - "
 
-	newres._description += " - ".join(descr_array)
+        newres._description += " - ".join(descr_array)
         newres.type = FuzzResult.result
 
-	return newres
+        return newres
 
     @staticmethod
     def from_baseline(fuzzresult, options):
-	scheme = fuzzresult.history.scheme
-	rawReq = str(fuzzresult.history)
-	auth_method, userpass = fuzzresult.history.auth
+        scheme = fuzzresult.history.scheme
+        rawReq = str(fuzzresult.history)
+        auth_method, userpass = fuzzresult.history.auth
 
         # get the baseline payload ordered by fuzz number and only one value per same fuzz keyword.
-        b1 = dict([matchgroup.groups() for matchgroup in re.finditer("FUZ(\d*)Z(?:\[.*?\])?(?:{(.*?)})?", rawReq, re.MULTILINE|re.DOTALL)])
-        b2 = dict([matchgroup.groups() for matchgroup in re.finditer("FUZ(\d*)Z(?:\[.*?\])?(?:{(.*?)})?", userpass, re.MULTILINE|re.DOTALL)])
-	baseline_control = dict(b1.items() + b2.items())
-        baseline_payload = map(lambda x: x[1], sorted(baseline_control.items(), key=operator.itemgetter(0)))
+        b1 = dict([matchgroup.groups() for matchgroup in re.finditer(r"FUZ(\d*)Z(?:\[.*?\])?(?:{(.*?)})?", rawReq, re.MULTILINE | re.DOTALL)])
+        b2 = dict([matchgroup.groups() for matchgroup in re.finditer(r"FUZ(\d*)Z(?:\[.*?\])?(?:{(.*?)})?", userpass, re.MULTILINE | re.DOTALL)])
+        baseline_control = dict(list(b1.items()) + list(b2.items()))
+        baseline_payload = [x[1] for x in sorted(list(baseline_control.items()), key=operator.itemgetter(0))]
 
-	# if there is no marker, there is no baseline request
-        if not filter(lambda x: x is not None, baseline_payload):
+        # if there is no marker, there is no baseline request
+        if not [x for x in baseline_payload if x is not None]:
             return None
 
-	## remove baseline marker from seed request
-	for i in baseline_payload:
+        # remove baseline marker from seed request
+        for i in baseline_payload:
             if not i:
                 raise FuzzExceptBadOptions("You must supply a baseline value for all the FUZZ words.")
-	    rawReq = rawReq.replace("{" + i + "}", '')
+            rawReq = rawReq.replace("{" + i + "}", '')
 
             if fuzzresult.history.wf_fuzz_methods:
                 fuzzresult.history.wf_fuzz_methods = fuzzresult.history.wf_fuzz_methods.replace("{" + i + "}", '')
@@ -575,15 +587,16 @@ class FuzzResultFactory:
             if auth_method:
                 userpass = userpass.replace("{" + i + "}", '')
 
-	# re-parse seed without baseline markers
-	fuzzresult.history.update_from_raw_http(rawReq, scheme)
-	if auth_method: fuzzresult.history.auth = (auth_method, userpass)
+        # re-parse seed without baseline markers
+        fuzzresult.history.update_from_raw_http(rawReq, scheme)
+        if auth_method:
+            fuzzresult.history.auth = (auth_method, userpass)
 
         # create baseline request from seed
         baseline_res = fuzzresult.from_soft_copy()
 
-	# remove field markers from baseline
-        marker_regex = re.compile("(FUZ\d*Z)\[(.*?)\]", re.DOTALL)
+        # remove field markers from baseline
+        marker_regex = re.compile(r"(FUZ\d*Z)\[(.*?)\]", re.DOTALL)
         results = marker_regex.findall(rawReq)
         if results:
             for fw, f in results:
@@ -599,42 +612,42 @@ class FuzzResultFactory:
 
         baseline_res = FuzzResultFactory.from_seed(baseline_res, baseline_payload, options)
 
-	baseline_res.is_baseline = True
+        baseline_res.is_baseline = True
         baseline_res.payload = baseline_payload
 
-	return baseline_res
+        return baseline_res
 
     @staticmethod
     def from_all_fuzz_request(seed, payload):
-	# only a fuzz payload is allowed using this technique
-	if len(payload) > 1:
-	    raise FuzzExceptBadOptions("Only one payload is allowed when fuzzing all parameters!")
+        # only a fuzz payload is allowed using this technique
+        if len(payload) > 1:
+            raise FuzzExceptBadOptions("Only one payload is allowed when fuzzing all parameters!")
 
-	for v in seed.history.wf_allvars_set:
-	    variable = v.name
-	    payload_content = payload[0]
+        for variable in seed.history.wf_allvars_set:
+            payload_content = payload[0]
             fuzzres = seed.from_soft_copy()
-	    fuzzres._description = variable + "=" + payload_content
+            fuzzres._description = variable.name + "=" + payload_content
             fuzzres.payload.append(payload_content)
 
-            fuzzres.history.wf_allvars_set = (variable, payload_content)
+            fuzzres.history.wf_allvars_set = (variable.name, payload_content)
 
-	    yield fuzzres
+            yield fuzzres
 
     @staticmethod
     def from_options(options):
-	fr = FuzzRequest()
+        fr = FuzzRequest()
 
-	fr.url = options['url']
-	fr.wf_fuzz_methods = options['method']
-	fr.update_from_options(options)
+        fr.url = options['url']
+        fr.wf_fuzz_methods = options['method']
+        fr.update_from_options(options)
+
+        return FuzzResult(fr)
 
-	return FuzzResult(fr)
 
 class MyCounter:
-    def __init__(self, count = 0):
+    def __init__(self, count=0):
         self._count = count
-	self._mutex = Lock()
+        self._mutex = Lock()
 
     def inc(self):
         self._operation(1)
@@ -650,57 +663,58 @@ class MyCounter:
         with self._mutex:
             return self._count
 
+
 class FuzzStats:
     def __init__(self):
-	self.mutex = Lock()
+        self.mutex = Lock()
 
-	self.url = ""
-	self.seed = None
+        self.url = ""
+        self.seed = None
 
-	self.total_req = 0
-	self.pending_fuzz = MyCounter()
-	self.pending_seeds = MyCounter()
-	self.processed = MyCounter()
-	self.backfeed = MyCounter()
-	self.filtered = MyCounter()
+        self.total_req = 0
+        self.pending_fuzz = MyCounter()
+        self.pending_seeds = MyCounter()
+        self.processed = MyCounter()
+        self.backfeed = MyCounter()
+        self.filtered = MyCounter()
 
-	self.totaltime = 0
-	self.__starttime = 0
+        self.totaltime = 0
+        self.__starttime = 0
 
-	self._cancelled = False
+        self._cancelled = False
 
     @staticmethod
     def from_requestGenerator(rg):
-	tmp_stats = FuzzStats()
+        tmp_stats = FuzzStats()
 
-	tmp_stats.url = rg.seed.history.redirect_url
-	tmp_stats.total_req = rg.count()
-	tmp_stats.seed = rg.seed
+        tmp_stats.url = rg.seed.history.redirect_url
+        tmp_stats.total_req = rg.count()
+        tmp_stats.seed = rg.seed
 
-	return tmp_stats
+        return tmp_stats
 
     def get_stats(self):
-	return {
-	    "url": self.url,
-	    "total": self.total_req,
+        return {
+            "url": self.url,
+            "total": self.total_req,
 
-	    "backfed": self.backfeed(),
-	    "Processed": self.processed(),
-	    "Pending": self.pending_fuzz(),
-	    "filtered": self.filtered(),
+            "backfed": self.backfeed(),
+            "Processed": self.processed(),
+            "Pending": self.pending_fuzz(),
+            "filtered": self.filtered(),
 
-	    "Pending_seeds": self.pending_seeds(),
+            "Pending_seeds": self.pending_seeds(),
 
-	    "totaltime": self._totaltime,
-	}
+            "totaltime": self._totaltime,
+        }
 
     def mark_start(self):
-	with self.mutex:
-	    self.__starttime = time.time()	
+        with self.mutex:
+            self.__starttime = time.time()
 
     def mark_end(self):
         with self.mutex:
-            self.totaltime = time.time() - self.__starttime	
+            self.totaltime = time.time() - self.__starttime
 
     @property
     def cancelled(self):
@@ -717,12 +731,12 @@ class FuzzStats:
 
         string += "Total time: %s\n" % str(self.totaltime)[:8]
 
-	if self.backfeed() > 0:
-	    string += "Processed Requests: %s (%d + %d)\n" % (str(self.processed())[:8], (self.processed() - self.backfeed()), self.backfeed())
-	else:
-	    string += "Processed Requests: %s\n" % (str(self.processed())[:8])
-	string += "Filtered Requests: %s\n" % (str(self.filtered())[:8])
-	string += "Requests/sec.: %s\n" % str(self.processed()/self.totaltime if self.totaltime > 0 else 0)[:8]
+        if self.backfeed() > 0:
+            string += "Processed Requests: %s (%d + %d)\n" % (str(self.processed())[:8], (self.processed() - self.backfeed()), self.backfeed())
+        else:
+            string += "Processed Requests: %s\n" % (str(self.processed())[:8])
+        string += "Filtered Requests: %s\n" % (str(self.filtered())[:8])
+        string += "Requests/sec.: %s\n" % str(self.processed() / self.totaltime if self.totaltime > 0 else 0)[:8]
 
         return string
 
@@ -737,21 +751,22 @@ class FuzzStats:
         self.filtered._operation(fuzzstats2.filtered())
         self.pending_seeds._operation(fuzzstats2.pending_seeds())
 
+
 class FuzzResult:
-    seed, backfeed, result, error, startseed, endseed, cancel, discarded = range(8)
-    newid = itertools.count(0).next
+    seed, backfeed, result, error, startseed, endseed, cancel, discarded = list(range(8))
+    newid = itertools.count(0)
     ERROR_CODE = -1
     BASELINE_CODE = -2
 
-    def __init__(self, history = None, exception = None, track_id = True):
-	self.history = history
+    def __init__(self, history=None, exception=None, track_id=True):
+        self.history = history
 
         self.type = None
-	self.exception = exception
+        self.exception = exception
         self._description = ""
         self.is_baseline = False
         self.rlevel = 1
-        self.nres = FuzzResult.newid() if  track_id else 0
+        self.nres = next(FuzzResult.newid) if track_id else 0
 
         self.chars = 0
         self.lines = 0
@@ -760,12 +775,12 @@ class FuzzResult:
 
         self.update()
 
-	self.plugins_res = []
-	self.plugins_backfeed = []
+        self.plugins_res = []
+        self.plugins_backfeed = []
 
         self.payload = []
 
-    def update(self, exception = None):
+    def update(self, exception=None):
         self.type = FuzzResult.result
 
         if exception:
@@ -774,12 +789,12 @@ class FuzzResult:
 
         if self.history and self.history.content:
             m = hashlib.md5()
-            m.update(self.history.content)
+            m.update(python2_3_convert_to_unicode(self.history.content))
             self.md5 = m.hexdigest()
 
             self.chars = len(self.history.content)
             self.lines = self.history.content.count("\n")
-            self.words = len(re.findall("\S+", self.history.content))
+            self.words = len(re.findall(r"\S+", self.history.content))
 
         return self
 
@@ -820,8 +835,8 @@ class FuzzResult:
     def code(self):
         if self.history and self.history.code >= 0 and not self.exception:
             return int(self.history.code)
-        #elif not self.history.code:
-            #return 0
+        # elif not self.history.code:
+            # return 0
         else:
             return FuzzResult.ERROR_CODE
 
@@ -838,18 +853,18 @@ class FuzzResult:
             raise FuzzExceptInternalError("A new seed cannot be created with a Fuzz item representing an error.")
 
         seed.history.url = self.history.recursive_url
-	seed.rlevel += 1
+        seed.rlevel += 1
         seed.type = FuzzResult.seed
 
-	return seed
+        return seed
 
-    def from_soft_copy(self, track_id = True):
-        fr = FuzzResult(self.history.from_copy(), track_id = track_id)
+    def from_soft_copy(self, track_id=True):
+        fr = FuzzResult(self.history.from_copy(), track_id=track_id)
 
-	fr.exception = self.exception
+        fr.exception = self.exception
         fr._description = self._description
         fr.is_baseline = self.is_baseline
-	fr.type = self.type
+        fr.type = self.type
         fr.rlevel = self.rlevel
         fr.payload = list(self.payload)
 
@@ -857,14 +872,14 @@ class FuzzResult:
 
     @staticmethod
     def to_new_exception(exception):
-        fr = FuzzResult(exception = exception, track_id = False)
+        fr = FuzzResult(exception=exception, track_id=False)
         fr.type = FuzzResult.error
 
         return fr
 
     @staticmethod
     def to_new_signal(signal):
-        fr = FuzzResult(track_id = False)
+        fr = FuzzResult(track_id=False)
         fr.type = signal
 
         return fr
@@ -872,37 +887,57 @@ class FuzzResult:
     def to_new_url(self, url):
         fr = self.from_soft_copy()
         fr.history.url = str(url)
-	fr._description = fr.history.path
-	fr.rlevel = self.rlevel + 1
+        fr._description = fr.history.path
+        fr.rlevel = self.rlevel + 1
         fr.type = FuzzResult.backfeed
         fr.is_baseline = False
 
         return fr
 
+    def __lt__(self, other):
+        return self.nres < other.nres
+
+    def __le__(self, other):
+        return self.nres <= other.nres
+
+    def __gt__(self, other):
+        return self.nres > other.nres
+
+    def __ge__(self, other):
+        return self.nres >= other.nres
+
+    def __eq__(self, other):
+        return self.nres == other.nres
+
+    def __ne__(self, other):
+        return self.nres != other.nres
+
+
 class PluginItem:
-    undefined, result, backfeed = range(3)
+    undefined, result, backfeed = list(range(3))
 
     def __init__(self, ptype):
-	self.source = ""
+        self.source = ""
         self.plugintype = ptype
 
+
 class PluginResult(PluginItem):
     def __init__(self):
         PluginItem.__init__(self, PluginItem.result)
 
-	self.issue = ""
+        self.issue = ""
+
 
 class PluginRequest(PluginItem):
     def __init__(self):
         PluginItem.__init__(self, PluginItem.backfeed)
 
-	self.fuzzitem = None
+        self.fuzzitem = None
 
     @staticmethod
     def from_fuzzRes(res, url, source):
-	plreq = PluginRequest()
-	plreq.source = source
-	plreq.fuzzitem = res.to_new_url(url)
-
-	return plreq
+        plreq = PluginRequest()
+        plreq.source = source
+        plreq.fuzzitem = res.to_new_url(url)
 
+        return plreq
