@@ -6,14 +6,17 @@ import tempfile
 import pipes
 import os
 
+
 @moduleman_plugin
 class screenshot(BasePlugin):
     name = "screenshot"
     author = ("Xavi Mendez (@xmendez)",)
     version = "0.1"
     summary = "Performs a screen capture using linux cutycapt tool"
-    description = ("Performs a screen capture using linux cutycapt tool",
-            "The tool must be installed and in the executable path",)
+    description = (
+        "Performs a screen capture using linux cutycapt tool",
+        "The tool must be installed and in the executable path",
+    )
     category = ["active"]
     priority = 99
 
@@ -22,9 +25,9 @@ class screenshot(BasePlugin):
 
     def __init__(self):
         BasePlugin.__init__(self)
-    
+
     def validate(self, fuzzresult):
-	return fuzzresult.code not in [404]
+        return fuzzresult.code not in [404]
 
     def process(self, fuzzresult):
         temp_name = next(tempfile._get_candidate_names())
@@ -32,5 +35,5 @@ class screenshot(BasePlugin):
 
         filename = os.path.join(defult_tmp_dir, temp_name + ".png")
 
-	subprocess.call(['cutycapt', '--url=%s' % pipes.quote(fuzzresult.url), '--out=%s' % filename])
-	self.add_result("Screnshot taken, output at %s" % filename)
+        subprocess.call(['cutycapt', '--url=%s' % pipes.quote(fuzzresult.url), '--out=%s' % filename])
+        self.add_result("Screnshot taken, output at %s" % filename)
